@@ -1,7 +1,7 @@
 // qsampler.cpp
 //
 /****************************************************************************
-   Copyright (C) 2004-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2016, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, 2008 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -33,13 +33,12 @@
 #include <QDir>
 #endif
 
-#define CONFIG_QUOTE1(x) #x
-#define CONFIG_QUOTED(x) CONFIG_QUOTE1(x)
+#ifndef CONFIG_PREFIX
+#define CONFIG_PREFIX	"/usr/local"
+#endif
 
-#if defined(DATADIR)
-#define CONFIG_DATADIR CONFIG_QUOTED(DATADIR)
-#else
-#define CONFIG_DATADIR CONFIG_PREFIX "/share"
+#ifndef CONFIG_DATADIR
+#define CONFIG_DATADIR	CONFIG_PREFIX "/share"
 #endif
 
 #if WIN32
@@ -204,10 +203,12 @@ public:
 		m_pWidget = pWidget;
 	#ifdef CONFIG_X11
 	#ifdef CONFIG_XUNIQUE
-		XGrabServer(m_pDisplay);
 		m_wOwner = m_pWidget->winId();
-		XSetSelectionOwner(m_pDisplay, m_aUnique, m_wOwner, CurrentTime);
-		XUngrabServer(m_pDisplay);
+		if (m_pDisplay && m_wOwner) {
+			XGrabServer(m_pDisplay);
+			XSetSelectionOwner(m_pDisplay, m_aUnique, m_wOwner, CurrentTime);
+			XUngrabServer(m_pDisplay);
+		}
 	#endif	// CONFIG_XUNIQUE
 	#endif	// CONFIG_X11
 	}
@@ -220,7 +221,7 @@ public:
 	{
 	#ifdef CONFIG_X11
 	#ifdef CONFIG_XUNIQUE
-		if (m_wOwner != None) {
+		if (m_pDisplay && m_wOwner != None) {
 			// First, notify any freedesktop.org WM
 			// that we're about to show the main widget...
 			Screen *pScreen = XDefaultScreenOfDisplay(m_pDisplay);
@@ -265,7 +266,7 @@ public:
 #ifdef CONFIG_XUNIQUE
 	void x11PropertyNotify(Window w)
 	{
-		if (m_pWidget && m_wOwner == w) {
+		if (m_pDisplay && m_pWidget && m_wOwner == w) {
 			// Always check whether our property-flag is still around...
 			Atom aType;
 			int iFormat = 0;
