@@ -1,7 +1,7 @@
 // qsamplerMessages.h
 //
 /****************************************************************************
-   Copyright (C) 2004-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2015, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -80,6 +80,9 @@ protected:
 	void appendMessagesLine(const QString& s);
 	void appendMessagesLog(const QString& s);
 
+	// Set stdout/stderr blocking mode.
+	bool stdoutBlock(int fd, bool bBlock) const;
+
 protected slots:
 
 	// Stdout capture slot.
